import { __awaiter } from 'tslib';
import { EventEmitter, Directive, ElementRef, Input, Output, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { debounceTime } from 'rxjs/operators';

class CanLoadDirective {
    constructor(elementRef) {
        this.elementRef = elementRef;
        this.throttlingTime = 1000;
        this.threshold = 1;
        this.intersectionChanged$ = new Subject();
        this.visible = new EventEmitter();
    }
    ngOnInit() {
        this.createObserver();
    }
    ngAfterViewInit() {
        this.startObserving();
    }
    /**
     * Check the visibility of listening(Child) element
     * @param {HTMLElement} element element listening to changes in scroll
     * @returns {boolean} if element is visible to the given extend then true, else false
     * @author Athul Narayanan
     */
    isVisible(element) {
        const option = {
            root: null,
            rootMargin: '0px',
            threshold: Number(this.threshold)
        };
        return new Promise((resolve, reject) => {
            const observer = new IntersectionObserver(([entry], observer) => {
                resolve(entry.intersectionRatio >= this.threshold);
                observer.disconnect();
            }, option);
            observer.observe(element);
        });
    }
    /**
     * Create intersection observer based on container element(usually the entire viewport)
     * @author Athul Narayanan
     */
    createObserver() {
        const option = {
            root: null,
            rootMargin: '0px',
            threshold: Number(this.threshold)
        };
        const isInterSecting = (entry) => entry.isIntersecting || entry.intersectionRatio >= this.threshold;
        this.observer = new IntersectionObserver((entries, observer) => {
            entries.forEach(entry => {
                var _a;
                if (isInterSecting(entry))
                    (_a = this.intersectionChanged$) === null || _a === void 0 ? void 0 : _a.next({ entry, observer });
            });
        }, option);
    }
    /**
     *
     * @returns an event saying that element is visble in the container
     * @author Athul Narayanan
     */
    startObserving() {
        var _a, _b;
        if (!this.observer)
            return;
        (_a = this.observer) === null || _a === void 0 ? void 0 : _a.observe(this.elementRef.nativeElement);
        (_b = this.intersectionChanged$) === null || _b === void 0 ? void 0 : _b.pipe(debounceTime(this.throttlingTime)).subscribe(({ entry, observer }) => __awaiter(this, void 0, void 0, function* () {
            var _c, _d;
            const isStillInThreshold = yield this.isVisible(entry.target);
            if (isStillInThreshold) {
                (_c = this.visible) === null || _c === void 0 ? void 0 : _c.emit(entry.target);
                (_d = this.observer) === null || _d === void 0 ? void 0 : _d.unobserve(entry.target);
            }
        }));
    }
}
CanLoadDirective.decorators = [
    { type: Directive, args: [{
                selector: '[canLoad]'
            },] }
];
CanLoadDirective.ctorParameters = () => [
    { type: ElementRef }
];
CanLoadDirective.propDecorators = {
    throttlingTime: [{ type: Input }],
    threshold: [{ type: Input }],
    visible: [{ type: Output }]
};

class NgxLazyLoadModule {
}
NgxLazyLoadModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    CanLoadDirective
                ],
                imports: [],
                exports: [
                    CanLoadDirective
                ]
            },] }
];

/*
 * Public API Surface of ngx-lazy-load
 */

/**
 * Generated bundle index. Do not edit.
 */

export { CanLoadDirective, NgxLazyLoadModule };
//# sourceMappingURL=ngx-lazy-load.js.map
